/*********************************************************************NVMH2****
Path:  C:\Dev\devrel\Nv_sdk_4\Dx8\NVEffectsBrowser\Effects\BM_DiffuseAndSpecular
File:  Effect_BM.cpp

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/


#include "eb_effect.h"
#include "nvdevice.h"
#include "nvmesh.h"

#include "Effect_BM.h"
#include "BM_Constants.h"
#include "NV_Error.h"
#include "NV_Common.h"
#include "SimpleObject8.h"


#define RIMPART 0
#define LUGSPART 1

#include <NVFileDialog.h>

// note diffuse is passed in as 3 floats?
#define WHEEL_FVF (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX5 | D3DFVF_TEXCOORDSIZE2(0) | \
		D3DFVF_TEXCOORDSIZE3(1) | D3DFVF_TEXCOORDSIZE3(2) | D3DFVF_TEXCOORDSIZE3(3) | \
		D3DFVF_TEXCOORDSIZE3(4))

using namespace std;

#define   STR_LOAD_NORMALMAP_TEXTURE	"Load Normal Map..."
#define   STR_LOAD_BASE_TEXTURE			"Load Base Texture..."
#define	  STR_LOADMESH					"Load Object..."
#define	  STR_LOAD_IRRADIANCE_MAP		"Load Irradiance Map..."

///////////////////////////////////////////////////////


DECLARE_EFFECT_MAIN()

extern "C"
{

__declspec(dllexport) unsigned int GetNumEffects() { return 1; }

__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
{
	switch(EffectNum)
	{
		case 0:
			return new Effect_BM();
		default:
			return NULL;
	}
}

}


#define STR_INCREASEBUMPSCALE "Bump scale increase (+)"
#define STR_DECREASEBUMPSCALE "Bump scale decrease (-)"




void Effect_BM::UpdateProperties()
{
	EBEffect::UpdateProperties();
	EBEnumProperty* pEnumProp = new EBEnumProperty("Display Options", OBJECT_MEMBER(m_eDisplayOption), EBTYPE_DWORD_PROP);
	
	pEnumProp->AddEnumerant(new EBEnumValue(pEnumProp, "Show Bump Map",
							(DWORD)DISPLAY_BUMPMAP, EBTYPE_DWORD_PROP));

	pEnumProp->AddEnumerant(new EBEnumValue(pEnumProp, "Show Texture Space Light Vector",
							(DWORD)DISPLAY_LIGHTVECTOR, EBTYPE_DWORD_PROP));

	pEnumProp->AddEnumerant(new EBEnumValue(pEnumProp, "Show Base Texture",
							(DWORD)DISPLAY_BASETEXTURE, EBTYPE_DWORD_PROP));

	pEnumProp->AddEnumerant(new EBEnumValue(pEnumProp, "Show Bumped Light + Base Texture",
							(DWORD)DISPLAY_RESULT, EBTYPE_DWORD_PROP));

	AddProperty(pEnumProp);


	AddProperty(new EBTriggerProperty(STR_LOAD_BASE_TEXTURE));
	AddProperty(new EBTriggerProperty(STR_LOAD_NORMALMAP_TEXTURE));
	AddProperty(new EBTriggerProperty(STR_LOAD_IRRADIANCE_MAP));


//gj  load mesh ability 
//	AddProperty(new EBTriggerProperty(STR_LOADMESH));



	// Vertex shaders
	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum,
			"Texture Space Calculation", GetFilePath("dot3_directional.nvv"), EBTYPE_STRING_PROP));

	AddProperty(new EBProperty("Wireframe", OBJECT_MEMBER(m_bWireframe), EBTYPE_BOOL_PROP));
	AddProperty(new EBProperty("Pause lights", OBJECT_MEMBER(m_bPause), EBTYPE_BOOL_PROP));

    string aboutText = GetFilePath("NVEffectsExplained.htm");
    aboutText += "#BrushedMetal";
	SetAboutInfo( NULL, _T("Brushed Metal"), _T(aboutText.c_str()));
	SetAboutInfo( NULL, _T("Kenneth Hurley - Developer Relations"), _T("http://www.nvidia.com/developer"));
	SetAboutInfo( NULL, _T("NVIDIA Corporation"), _T("http://www.nvidia.com"));
	SetAboutInfo( _T("Date"), _T("May 2001"));
}


void Effect_BM::LoadTexture( LPDIRECT3DTEXTURE8 * ppTex, const EBString & filename )
{
	HRESULT hr;

	if( *ppTex != NULL )
	{
		SAFE_RELEASE( *ppTex );
	}

	//load texture
	hr = D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath(filename.c_str()).c_str(), ppTex );

}



void Effect_BM::PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten)
{
	if (!bWritten)
		return;

	EBString	name;

	if (pProperty->IsKindOf(EBTYPE_TRIGGER_PROP))
	{

		name = pProperty->GetPropertyName();

		if( name == EBString( STR_LOAD_NORMALMAP_TEXTURE ) )
		{
			// Present a file dialog
			NVTextureFileDialog aDialog;

			std::string theResult;
			bWritten = ( aDialog.Open( theResult ) );

			m_strBumpMapFileName = theResult;

			if ( bWritten )
			{
				LoadTexture( &m_pBumpMap, m_strBumpMapFileName );
				FDebug("Loaded new bump map texture!  %s\n", m_strBumpMapFileName.c_str() );

			}
		}
		else if( name == EBString( STR_LOAD_BASE_TEXTURE ) )
		{
			// Present a file dialog
			NVTextureFileDialog aDialog;
			std::string theResult;
			bWritten = ( aDialog.Open( theResult ) );
			m_strBaseTexFileName = theResult;
			if ( bWritten )
			{
				LoadTexture( &m_pBaseTexture, m_strBaseTexFileName );
				FDebug("Loaded new base texture:    %s\n", m_strBaseTexFileName.c_str() );
			}
		}
		else if( name == EBString( STR_LOAD_IRRADIANCE_MAP ))
		{
			// Present a file dialog
			NVTextureFileDialog aDialog;
			std::string theResult;
			bWritten = ( aDialog.Open( theResult ) );
			m_strIrradianceMapFileName = theResult;
			if ( bWritten )
			{
				LoadTexture( &m_pTex_DiffSpec_IrradianceMap, m_strIrradianceMapFileName );
				FDebug("Loaded new illumination map texture:   %s\n", m_strIrradianceMapFileName.c_str() );
			}
		}
	}
}



Effect_BM::Effect_BM()
:	m_eDisplayOption(DISPLAY_RESULT),
	m_fAngle(0.0f),
	m_pLightMesh(NULL),
	m_pWheelMesh(NULL),
	m_pNVDevice(NULL),
	m_pBumpMap(NULL),
	m_pBaseTexture(NULL),
	m_pTex_White(NULL),
	m_pTex_DiffSpec_IrradianceMap(NULL),
	m_dwLightShader(0),
	m_dwDot3Shader(0),
	m_dwTextureShader(0),
	m_pUI(NULL),
	m_bWireframe(false),
	m_bPause(false),
	m_fBumpScale(.3f)
{
	m_strEffectName = "Brushed Metal2";			// A string holding the name of the effect
	m_strEffectLocation = "Effects\\Bump Mapping";
	m_strEffectPixelShader = "";
	m_strEffectVertexShader = GetFilePath("dot3_directional.nvv");
	m_strEffectVersion = "2.0";

	m_dwPSH_BumpDiffSpec = NULL;
	m_dwPSH_ShowLightVector = NULL;
	m_dwPSH_Textured = NULL;

}



Effect_BM::~Effect_BM()
{	
	Free();	
}


//////////////////////////////////////////////////////////////////////////////////////

#define TEX_SCALE 1.0f


void	Effect_BM::ReleaseTextures()
{
	SAFE_RELEASE(m_pBumpMap);
	SAFE_RELEASE(m_pBaseTexture);
	SAFE_RELEASE( m_pTex_DiffSpec_IrradianceMap  );
	SAFE_RELEASE( m_pTex_White);

}


void	Effect_BM::ReloadTextures()
{
	HRESULT hr;

	ReleaseTextures();


	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath(m_strBumpMapFileName.c_str()).c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		D3DX_DEFAULT,						// mip levels
		0,
		D3DFMT_A8R8G8B8,
		D3DPOOL_DEFAULT,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_pBumpMap);
	if (FAILED(hr))
	{
		m_strLastError = "Could not create bump map texture";
	}


	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath(m_strBaseTexFileName.c_str()).c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		D3DX_DEFAULT,						// mip levels
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_DEFAULT,
		D3DX_FILTER_LINEAR,
		D3DX_FILTER_LINEAR,
		0,
		NULL,
		NULL,
		&m_pBaseTexture);
	if (FAILED(hr))
	{
		m_strLastError = "Could not create base texture";
	}

	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath(m_strIrradianceMapFileName.c_str()).c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_A8R8G8B8,
		D3DPOOL_DEFAULT,
		D3DX_FILTER_LINEAR | D3DX_FILTER_MIRROR,	// cannot wrap when creating mip levels
		D3DX_FILTER_LINEAR | D3DX_FILTER_MIRROR,
		0,
		NULL,
		NULL,
		&m_pTex_DiffSpec_IrradianceMap );
	if (FAILED(hr))
	{
		m_strLastError = "Could not create diffuse specular gradient texture";
	}

	hr = D3DXCreateTextureFromFileEx(m_pD3DDev, 
		GetFilePath("white.dds").c_str(),
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		0,
		D3DFMT_A8R8G8B8,
		D3DPOOL_DEFAULT,
		D3DX_FILTER_LINEAR | D3DX_FILTER_MIRROR,	// cannot wrap when creating mip levels
		D3DX_FILTER_LINEAR | D3DX_FILTER_MIRROR,
		0,
		NULL,
		NULL,
		&m_pTex_White );
	if (FAILED(hr))
	{
		m_strLastError = "Could not create diffuse specular gradient texture";
	}
}




HRESULT Effect_BM::Initialize(IDirect3DDevice8* pDev)
{
	HRESULT hr;
	vector<DWORD> Declaration;

	LPDIRECT3DVERTEXBUFFER8 pVB;
	LPDIRECT3DINDEXBUFFER8 pIB;

	//////////////////////////////////////////////////////////

	m_strBumpMapFileName = "BrushedMetalBM.dds";
	m_strBaseTexFileName = "silver.tga";
	m_strIrradianceMapFileName = "SilverGradient.tga";

	//////////////////////////////////////////////////////////

	m_pD3DDev = pDev;
	pDev->AddRef();


	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	D3DVIEWPORT8 viewport;
	m_pD3DDev->GetViewport(&viewport);
	rect.bottom = viewport.Height;
	rect.right  = viewport.Width;
	m_pUI = new MouseUI((const RECT)rect);

	m_pNVDevice = new NVDot3DirectionalDevice(pDev, this);

	m_pLightMesh = new NVMesh();
	hr = m_pLightMesh->Create(m_pNVDevice, GetFilePath("arrow.x"));
	if (FAILED(hr))
	{
		m_strLastError = "Could not load arrow.x";
		return hr;
	}
	m_pLightMesh->SetFVF(m_pNVDevice, D3DFVF_NORMAL | D3DFVF_XYZ | D3DFVF_TEX1);


	///////////////////////////////////////////////////////////////////////////////////

	m_pWheelMesh = new NVMesh();
	hr = m_pWheelMesh->Create(m_pNVDevice, GetFilePath("wheel.x"));
	if (FAILED(hr))
	{
		m_strLastError = "Could not load wheel.x";
		return hr;
	}

	hr = m_pWheelMesh->SetFVF(m_pNVDevice, WHEEL_FVF);
	if (FAILED(hr))
	{
		m_strLastError = "Could not convert FVF";
		return hr;
	}

	hr = m_pWheelMesh->GetSysMemMesh()->GetVertexBuffer(&pVB);
	if (FAILED(hr))
	{
		m_strLastError = "Failed to get Vertex Buffer";
		return hr;
	}
	hr = m_pWheelMesh->GetSysMemMesh()->GetIndexBuffer(&pIB);
	if (FAILED(hr))
	{
		m_strLastError = "Failed to get Index Buffer";
		return hr;
	}

	// move the texture coordinates from the diffuse component to the actual
	// texture coordinates.  Quick hack to get the FVF into our format
	// Calculate number of vertices and indices
	D3DVERTEXBUFFER_DESC VBDesc;
	D3DINDEXBUFFER_DESC IBDesc;
	
	pVB->GetDesc(&VBDesc);
	pIB->GetDesc(&IBDesc);

	DWORD dwNumIndices;
	DWORD dwNumVertices;
	switch(IBDesc.Format)
	{
		case D3DFMT_INDEX16:
			dwNumIndices = IBDesc.Size / 2;
			break;
		case D3DFMT_INDEX32:
			dwNumIndices = IBDesc.Size / 4;
			break;
		default:
			assert(0);
			return E_FAIL;
	}

	dwNumVertices = VBDesc.Size / sizeof(Dot3Vertex);

	Dot3Vertex* pVertices;
	WORD* pIndices;

	// Get a pointer to the indices and the vertices
	hr = pVB->Lock(0, 0, (BYTE**)&pVertices, 0);
	if (FAILED(hr))
		return hr;

	hr = pIB->Lock(0, 0, (BYTE**)&pIndices, 0);
	if (FAILED(hr))
		return hr;

	DWORD i;
	// Clear the basis vectors
	for (i = 0; i < dwNumVertices; i++)
	{
		pVertices[i].Texture.x = pVertices[i].Diffuse.x;
		pVertices[i].Texture.y = pVertices[i].Diffuse.y;
		pVertices[i].Diffuse = D3DXVECTOR3(1.0f, 1.0f, 1.0f);
	}

	pVB->Unlock();
	pIB->Unlock();

	hr = CreateBasisVectors(pVB, pIB);
	if (FAILED(hr))
	{
		m_strLastError = "Failed to create basis vectors";
		return hr;
	}


	///////////////////////////////////////////////////////////////////////////////////


	// Create the shaders
	Declaration.clear();
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3)); // Pos
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3)); // Normal
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT2)); // Texture
	Declaration.push_back(D3DVSD_END());
	hr = LoadAndCreateShader(GetFilePath("dot3_lightobject.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwLightShader);
	if (FAILED(hr))
		return hr;

	m_pLightMesh->SetVertexShader(m_dwLightShader);

	Declaration.clear();
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3)); // Position
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3)); // Normal
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT3)); // Diffuse
	Declaration.push_back(D3DVSD_REG(3, D3DVSDT_FLOAT2)); // Texture
	Declaration.push_back(D3DVSD_REG(5, D3DVSDT_FLOAT3)); // S
	Declaration.push_back(D3DVSD_REG(6, D3DVSDT_FLOAT3)); // T
	Declaration.push_back(D3DVSD_REG(4, D3DVSDT_FLOAT3)); // SxT
	Declaration.push_back(D3DVSD_END());
	hr = LoadAndCreateShader(GetFilePath("dot3_directional.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwDot3Shader);
	if (FAILED(hr))
		return hr;

	hr = LoadAndCreateShader(GetFilePath("vs_textured.vso"), &Declaration[0], 0, SHADERTYPE_VERTEX, &m_dwTextureShader);
	if (FAILED(hr))
		return hr;

	m_pWheelMesh->SetVertexShader(m_dwTextureShader);

	hr = LoadAndCreateShader(GetFilePath("ps_dot3.pso"), 0, 0, SHADERTYPE_PIXEL,
								&m_dwPSH_BumpDiffSpec );
	ASSERT_AND_RET_IF_FAILED( hr );


	hr = LoadAndCreateShader(GetFilePath("ps_ShowLightVector.pso"), 0, 0, SHADERTYPE_PIXEL,
								&m_dwPSH_ShowLightVector );
	ASSERT_AND_RET_IF_FAILED(hr);


	hr = LoadAndCreateShader(GetFilePath("ps_textured.pso"), 0, 0, SHADERTYPE_PIXEL,
								&m_dwPSH_Textured );
	ASSERT_AND_RET_IF_FAILED(hr);

	ReloadTextures();



	// Setup constants
	m_pD3DDev->SetVertexShaderConstant(CV_ZERO, D3DXVECTOR4(0.0f, 0.0f, 0.0f, 0.0f), 1);
	m_pD3DDev->SetVertexShaderConstant(CV_ONE, D3DXVECTOR4(1.0f, 1.0f, 1.0f, 1.0f), 1);
	m_pD3DDev->SetVertexShaderConstant(CV_HALF, D3DXVECTOR4(0.5f, 0.5f, 0.5f, 0.5f), 1);

	// Camera stuff
	D3DXMATRIX matProj;
	D3DXMATRIX matView;

	m_InitialEyePos = D3DXVECTOR3( 0.0f, 0.0f, -350.0f );

	D3DXVECTOR3 vEyePt	  = m_InitialEyePos;

	D3DXVECTOR4 tmp;
	tmp.x = vEyePt.x;
	tmp.y = vEyePt.y;
	tmp.z = vEyePt.z;
	tmp.w = 0.0f;
	
	// -3.5
	m_pD3DDev->SetVertexShaderConstant(CV_EYE_POS_WORLD, tmp, 1);


	D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUp		  = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

	// View
	D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUp);



	// Projection
	D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(60.0f), 1.0f, 1.0f, 2500.0f);

	m_pNVDevice->SetViewTransform(&matView);
	m_pNVDevice->SetProjectionTransform(&matProj);

	m_pD3DDev->SetRenderState(D3DRS_SPECULARENABLE, FALSE);


	if (m_pLightMesh)
		m_pLightMesh->RestoreDeviceObjects(m_pNVDevice);

	if (m_pWheelMesh)
		m_pWheelMesh->RestoreDeviceObjects(m_pNVDevice);

	return S_OK;

}

HRESULT Effect_BM::Free()
{
	DWORD i;

	for (i=0; m_pWheelMesh && i<m_pWheelMesh->m_dwNumMaterials; i++ )
	{
		
		if (m_pWheelMesh->m_pTextures[i] == m_pTex_White)
		{
			// kill white texture in material list
			m_pWheelMesh->m_pTextures[i] = NULL;
		}
	}

	SAFE_DELETE(m_pLightMesh);
	SAFE_DELETE(m_pWheelMesh);

	SAFE_DELETE(m_pNVDevice);

	ReleaseTextures();

	SAFE_DELETE( m_pUI );

	if (m_pD3DDev)
	{
		if (m_dwDot3Shader)
		{
			m_pD3DDev->DeleteVertexShader(m_dwDot3Shader);
			m_dwDot3Shader = NULL;
		}

		if (m_dwLightShader)
		{
			m_pD3DDev->DeleteVertexShader(m_dwLightShader);
			m_dwDot3Shader = NULL;
		}

		if (m_dwTextureShader)
		{
			m_pD3DDev->DeleteVertexShader(m_dwTextureShader);
			m_dwTextureShader = NULL;
		}

		if (m_dwPSH_BumpDiffSpec )
		{
			m_pD3DDev->DeletePixelShader( m_dwPSH_BumpDiffSpec );
			m_dwPSH_BumpDiffSpec = NULL;
		}

		if (m_dwPSH_ShowLightVector)
		{
			m_pD3DDev->DeletePixelShader( m_dwPSH_ShowLightVector );
			m_dwPSH_ShowLightVector = NULL;
		}

		if (m_dwPSH_Textured)
		{
			m_pD3DDev->DeletePixelShader( m_dwPSH_Textured );
			m_dwPSH_Textured = NULL;
		}


		m_dwPSH_BumpDiffSpec		= 0;
		m_dwPSH_ShowLightVector		= 0;
		m_dwLightShader				= 0;
		m_dwDot3Shader				= 0;

		SAFE_RELEASE(m_pD3DDev);
	}
	
	return S_OK;
}

HRESULT Effect_BM::Start()
{
	m_fAngle = 0.0f;
	return S_OK;
}

  
HRESULT Effect_BM::SetTransform()
{
	D3DXMATRIX matTemp;
	D3DXMATRIX matWorldViewProj;
	D3DXMATRIX matWorldView;
	D3DXMATRIX matWorldViewIT;
	D3DXMATRIX matWorldInverse;

	D3DXMATRIX matWorld = m_pNVDevice->GetWorldTransform();
	D3DXMATRIX matProj = m_pNVDevice->GetProjectionTransform();
	D3DXMATRIX matView = m_pNVDevice->GetViewTransform();

	D3DXMatrixMultiply(&matTemp, &matWorld, &matView);
	D3DXMatrixMultiply(&matWorldViewProj, &matTemp, &matProj);
	D3DXMatrixMultiply(&matWorldView, &matWorld, &matView);
	D3DXMatrixInverse(&matWorldInverse, NULL, &matWorld);
	
	// Create a 3x3 inverse of the worldview for the normal transformation (we transpose it as we load it into
	// the constant store)
	D3DXMatrixInverse(&matWorldViewIT, NULL, &matWorldView);
		
	// Projection to clip space
	D3DXMatrixTranspose(&matWorldViewProj, &matWorldViewProj);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDVIEWPROJ_0, &matWorldViewProj(0, 0), 4);
	D3DXMatrixTranspose(&matWorldViewProj, &matWorldViewProj);

	// Transform to eye space
	D3DXMatrixTranspose(&matWorldView, &matWorldView);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDVIEW_0, &matWorldView(0, 0), 4);
	D3DXMatrixTranspose(&matWorldView, &matWorldView);

	// Transform to world space
	D3DXMatrixTranspose(&matWorld, &matWorld);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLD_0, &matWorld(0, 0), 4);
	D3DXMatrixTranspose(&matWorld, &matWorld);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDOBJECT_0, &matWorld(0, 0), 4);

	// Transform for normals
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDVIEWIT_0, &matWorldViewIT(0, 0), 1);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDVIEWIT_1, &matWorldViewIT(1, 0), 1);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDVIEWIT_2, &matWorldViewIT(2, 0), 1);

	return S_OK;

}



// Draw the actual light so we can see where it's pointing.
// vecLightDirection sets distance off of the object.
// It is used to create a normalized vector for illumination

HRESULT Effect_BM::DrawDirectionalLightObject(D3DXVECTOR3 vecLightDirection)
{
	D3DXMATRIX matWorld;
	D3DXMATRIX matTemp;
	D3DXVECTOR3 vecNormalizedDirection;

	m_pD3DDev->SetPixelShader(0);

	const NVBounds* pBounds = m_pWheelMesh->GetBounds();
	const NVBounds* pBoundsLight = m_pLightMesh->GetBounds();
	D3DXMatrixIdentity(&matWorld);

	// Translate to the middle
	D3DXMatrixTranslation(&matTemp, -pBoundsLight->m_vecCenter.x, 
		-pBoundsLight->m_vecCenter.y, -pBoundsLight->m_vecCenter.z);
	D3DXMatrixMultiply(&matWorld, &matWorld, &matTemp);

	float fLightScale = 30.0f;
	D3DXMatrixScaling(&matTemp, fLightScale / pBounds->m_fRadius, fLightScale / pBounds->m_fRadius, fLightScale / pBounds->m_fRadius);
	D3DXMatrixMultiply(&matWorld, &matWorld, &matTemp);

	// Orientation of the light object (the arrow)
	D3DXVECTOR3 vLightObject = D3DXVECTOR3(0.0f, -1.0f, 0.0f);

	D3DXVec3Normalize(&vecNormalizedDirection, &vecLightDirection);

	// Create the rotation axis
	D3DXVECTOR3 vOrthoNormal;
	D3DXVec3Cross(&vOrthoNormal, &vecNormalizedDirection, &vLightObject);
	D3DXVec3Normalize(&vOrthoNormal, &vOrthoNormal);

	// Calculate the angle between the two vectors.
	float fAngle = acos(D3DXVec3Dot(&vecNormalizedDirection, &vLightObject));

	// Rotate the object about our rotation axis to map one vector onto another
	D3DXMatrixRotationAxis(&matTemp, &vOrthoNormal, -fAngle);
	D3DXMatrixMultiply(&matWorld, &matWorld, &matTemp);

	D3DXMATRIX matTranslate = m_pUI->GetTranslationMatrix();
	// vecLightDirection is a direction vector, pointing at the origin, so we negate it to find the position
	D3DXMatrixTranslation(&matTemp, -vecLightDirection.x + matTranslate._41, -vecLightDirection.y + matTranslate._42, -vecLightDirection.z + matTranslate._43);

	D3DXMatrixMultiply(&matWorld, &matWorld, &matTemp);

	D3DXVECTOR3 LightColor(1.0f, 1.0f, 1.0f);
	m_pD3DDev->SetVertexShaderConstant(CV_LIGHT_COLOR, &LightColor.x, 1);

	m_pNVDevice->SetWorldTransform(&matWorld);
	SetTransform();

	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_DISABLE);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);

	m_pLightMesh->Render(m_pNVDevice);		

	return S_OK;
}


HRESULT Effect_BM::Tick(EBTimer* pTimer)
{
	HRESULT hr = S_OK;
	D3DXMATRIX matWorld;
	D3DXMATRIX matView = m_pNVDevice->GetViewTransform();
	D3DXMATRIX matTemp;

	// Clear to grey
	hr = m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
						D3DCOLOR_XRGB( 0xAA, 0xAA, 0xAA ), 1.0, 0);

	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID);
	// Increase rotation
	
	if (!m_bPause)
		m_fAngle += 0.1f;

	// World
	D3DXMatrixMultiply( &matWorld, &m_pUI->GetRotationMatrix(), &m_pUI->GetTranslationMatrix() );
			
	// Load transform data.
	m_pNVDevice->SetWorldTransform(&matWorld);
	SetTransform();

	m_pD3DDev->SetVertexShaderConstant(CV_BUMP_SCALE, D3DXVECTOR4(0.0f, 0.0f, 0.0f, m_fBumpScale), 1);

	// Create the light direction vector.
	D3DXVECTOR3 vecLightDirection;

	float wheelRad = m_pWheelMesh->GetBounds()->m_fRadius;
	float fLightDistance = wheelRad * 1.1f;

	vecLightDirection.x = fLightDistance * cos( D3DXToRadian(m_fAngle));
	vecLightDirection.y = fLightDistance * sin( D3DXToRadian(m_fAngle));
	vecLightDirection.z =  1.1f;

	//FDebug("light:  %f   %f   %f\n", vecLightDirection.x, vecLightDirection.y, vecLightDirection.z );

	// Light direction is normalized in the DrawObjects() function

	// Do the rendering
	DrawObjects(vecLightDirection);

	// Draw the light shape
	DrawDirectionalLightObject(vecLightDirection);

	return hr;
}




HRESULT Effect_BM::DrawObjects(D3DXVECTOR3 vecLightDirection)
{
	D3DXMATRIX matWorld;
	D3DXMATRIX matTemp;
	LPDIRECT3DVERTEXBUFFER8 pVB;
	LPDIRECT3DINDEXBUFFER8 pIB;
	LPD3DXMESH wheelMesh;

	D3DXVec3Normalize(&vecLightDirection, &vecLightDirection);

	// shader math requires that the light direction vector points towards the light.
	vecLightDirection = -vecLightDirection;

	for (DWORD i=0; i<m_pWheelMesh->m_dwNumMaterials; i++ )
	{
		
		if ((i != RIMPART) && (i != LUGSPART))
		{
			if (m_pWheelMesh->m_pTextures[i] == NULL)
			{
				// use textured shader with white texture
				m_pWheelMesh->m_pTextures[i] = m_pTex_White;
			}

			m_pD3DDev->SetPixelShader(m_dwPSH_Textured);
			m_pWheelMesh->RenderPart(m_pNVDevice, i);
			continue;
		}

		D3DXATTRIBUTERANGE* pAttrib = &m_pWheelMesh->m_pAttributeTable[i];
		if (!pAttrib)
			return E_FAIL;

		m_pD3DDev->SetTexture(0, m_pBaseTexture );
		m_pD3DDev->SetTexture(1, m_pBumpMap );
		m_pD3DDev->SetTexture(3, m_pTex_DiffSpec_IrradianceMap );
		// Override the setup options if the user wants to display the components
		switch(m_eDisplayOption)
		{
			default:
			case DISPLAY_RESULT:

				m_pD3DDev->SetPixelShader( m_dwPSH_BumpDiffSpec );
				break;

			case DISPLAY_BUMPMAP:
				// Show the bump texture, disable the base texture pass

				m_pD3DDev->SetPixelShader( 0 );

				m_pD3DDev->SetTexture( 0, m_pBumpMap );
				m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE		);
				m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1	);
				m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE		);
				m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE		);
				break;

			case DISPLAY_LIGHTVECTOR:
				// Show the light vector which, for this example, the vertex shader 
				//  writes to the t2 texture coordinate.

				m_pD3DDev->SetPixelShader( m_dwPSH_ShowLightVector );
				break;

			case DISPLAY_BASETEXTURE:

				m_pD3DDev->SetPixelShader( 0 );

				m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE		);
				m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1	);
				m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE		);
				m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_DISABLE		);
				break;
		}


		// Base texture stage
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
		m_pD3DDev->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);
		m_pD3DDev->SetRenderState( D3DRS_WRAP0, 0 );

		// bump map texture stage
		m_pD3DDev->SetTextureStageState(1, D3DTSS_ADDRESSU,	D3DTADDRESS_WRAP);
		m_pD3DDev->SetTextureStageState(1, D3DTSS_ADDRESSV,	D3DTADDRESS_WRAP);
		m_pD3DDev->SetRenderState( D3DRS_WRAP1, 0 );


		// Illumination texture
		m_pD3DDev->SetTextureStageState(2, D3DTSS_ADDRESSU,	D3DTADDRESS_CLAMP);
		m_pD3DDev->SetTextureStageState(2, D3DTSS_ADDRESSV,	D3DTADDRESS_CLAMP);	
		m_pD3DDev->SetTextureStageState(2, D3DTSS_ADDRESSW,	D3DTADDRESS_CLAMP);
		m_pD3DDev->SetRenderState( D3DRS_WRAP2, 0);

		m_pD3DDev->SetTextureStageState(3, D3DTSS_ADDRESSU,	D3DTADDRESS_CLAMP);
		m_pD3DDev->SetTextureStageState(3, D3DTSS_ADDRESSV,	D3DTADDRESS_CLAMP);	
		m_pD3DDev->SetTextureStageState(3, D3DTSS_ADDRESSW,	D3DTADDRESS_CLAMP);	


		// Need wrap for the sphere model - no dup verts in the sphere
		m_pD3DDev->SetRenderState( D3DRS_WRAP3, 0);


		// Load the location of the light into the vertex shader.
		m_pD3DDev->SetVertexShaderConstant(CV_LIGHT_DIRECTION, (void*)&vecLightDirection.x, 1);
		
		// Setup the vertex shader
		m_pD3DDev->SetVertexShader(m_dwDot3Shader);

		// Backface cull the sphere
		m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

		wheelMesh = m_pWheelMesh->GetLocalMesh();

		wheelMesh->GetVertexBuffer(&pVB);
		wheelMesh->GetIndexBuffer(&pIB);

		// Setup the stream and index buffers for the mesh
		m_pD3DDev->SetStreamSource(0, pVB, sizeof(Dot3Vertex));
		m_pD3DDev->SetIndices(pIB, 0);

		// Draw it.
		m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, pAttrib->VertexStart, pAttrib->VertexCount, pAttrib->FaceStart * 3, pAttrib->FaceCount);
	}

	return S_OK;
}


HRESULT Effect_BM::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	if (!(pCaps->MaxSimultaneousTextures >= 2))
	{
		m_strLastError = "Device cannot dual texture!";
		return E_FAIL;
	}

	if (!(pCaps->TextureOpCaps & D3DTEXOPCAPS_DOTPRODUCT3))
	{
		m_strLastError = "Device cannot handle dotproduct3 operation!";
		return E_FAIL;
	}

	if(D3DSHADER_VERSION_MAJOR( pCaps->PixelShaderVersion) < 1)
	{
		m_strLastError = "Device does not support pixel shaders!";
		return E_FAIL;
	}

	if(D3DSHADER_VERSION_MAJOR( pCaps->PixelShaderVersion) < 1)
	{
		m_strLastError = "Device does not support pixel shaders!";
		return E_FAIL;
	}

	return S_OK;
}


void Effect_BM::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if(button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_pUI->OnLButtonDown(x, y);
		}
		else
		{
			m_pUI->OnLButtonUp(x, y);
		}
	}
	return;
}



void Effect_BM::MouseMove(HWND hWnd, int x, int y)
{
	m_pUI->OnMouseMove(x, y);

	// Need to update eye position for specular calc,
	//  so get translation amount from MouseUI

	D3DXMATRIX tmat;
	m_pUI->GetTranslationMatrix( &tmat );

	// Get translation amount:
	float xx,yy,zz;

	xx = tmat._41;
	yy = tmat._42;
	zz = tmat._43;

	//FDebug("x  %f  y %f  z %f\n", xx,yy,zz );

	// Add in the initial offset.  Coordinates in the MouseUI differ
	//  from the world space by this amount

	xx += m_InitialEyePos.x;
	yy += m_InitialEyePos.y;
	zz += m_InitialEyePos.z;

	m_pD3DDev->SetVertexShaderConstant(CV_EYE_POS_WORLD, D3DXVECTOR4( -xx, -yy, zz, 0.0f ), 1 );

	return;
}



void Effect_BM::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);

	
	if( bDown )
	{
		switch( dwKey )
		{
		case 'R':
			ReloadTextures();
			break;
		}

	}



    switch ( Action )
    {
		case EB_HELP:
		{
			::MessageBoxEx( NULL, " Help : F1 - Help \n\n Home - Reset To Defaults \n\n W - Wireframe Toggle \n\n Space\\Pause - Toggle Pause/Resume \n\n Left Button & Mouse - Rotate Object\n\n Shift Left Button & Mouse - Pan Camera \n\n Ctrl Left Button & Mouse - Move Camera In & Out\n"
				STR_INCREASEBUMPSCALE "\n" STR_DECREASEBUMPSCALE "\n\n",
				   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
		}
		break;

		case EB_PAUSE:
		{
			m_bPause = !m_bPause;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
		}
		break;

		case EB_WIREFRAME:
        {
            m_bWireframe = !m_bWireframe;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

		case EB_RESET:
        {
            m_pUI->Reset();
            m_bWireframe = false;
			m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
            
        }
		break;

		case EB_SUBTRACT:
		{
			m_fBumpScale -= .1f;
		}
		break;

		case EB_ADD:
		{
			m_fBumpScale += .1f;
		}
		break;

        default :
            break;
    }
}